'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

// check whether it is an object or not
var isObject = exports.isObject = function isObject(obj) {
  return obj !== null && ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' || typeof obj === 'function');
};

var isFunction = exports.isFunction = function isFunction(fn) {
  return typeof fn === 'function';
};

// map through all object keys and run a function
var forEach = exports.forEach = function forEach(obj, fn) {

  var propertyNames = isObject(obj) ? Object.keys(obj) : [];

  propertyNames.map(function (propertyName) {
    fn(obj[propertyName], propertyName);
  });
};

// return the shaper function as string based on given schema
var shaperFunction = exports.shaperFunction = function shaperFunction(schema) {

  var functionCode = '';

  // start recursion
  recursiveWalk(schema);

  // write function code recursively
  function recursiveWalk(current, propertyName) {

    // run extraction function if found
    if (isFunction(current)) {
      // remove istanbul coverage code
      var cleanFunctionCode = current.toString().replace(/cov_\w+\.\w.*?[,;]/g, ' ');

      functionCode += '"' + propertyName + '":  ' + cleanFunctionCode + '(data), ';
    }
    // if it's an object
    else if (isObject(current)) {

        // it's not the root object
        if (propertyName) {
          // so write the property name
          functionCode += '"' + propertyName + '": ';
        }

        // open object
        functionCode += '{ ';

        // go through all properties and run recursion
        forEach(current, function (nextProperty, nextPropertyName) {
          recursiveWalk(nextProperty, nextPropertyName);
        });

        // close object
        functionCode += ' }';

        // only add comma if it's not the root object
        if (propertyName) {
          functionCode += ', ';
        }
      }
      // otherwise it's a property only
      else {
          // assign value
          functionCode += '"' + propertyName + '":  data[ "' + current + '" ], ';
        }
  }

  // return the function as string
  return functionCode;
};

// return the shaper function as executable function based on given schema
var shaper = exports.shaper = function shaper(schema) {

  // generate function
  var fnString = shaperFunction(schema);

  // generate executable function
  return new Function('data', 'require', 'return ' + fnString);
};

// default export
exports.default = shaper;
